/* 
  WiFiTelnetToSerial - Example Transparent UART to Telnet Server for esp8266

  Copyright (c) 2015 Hristo Gochkov. All rights reserved.
  This file is part of the ESP8266WiFi library for Arduino environment.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <ESP8266WiFi.h>
#include <algorithm>

//Set to 1 for the server to loop messages back to client
#define COMM_LOOP 0

//Set to 1 for Serial Loop (same idea as above)
#define SERIAL_LOOP 1

//Set to 1 for debug prints on start
#define DEBUG_PRINT 1

//how many clients should be able to telnet to this ESP8266
#define MAX_SRV_CLIENTS 1

//SSID and PASSWORD SAME AS MICROCART WIFI IN LAB OR ON AP ESP8266 ESP-01
const char* ssid = "MicroCART";
const char* password = "m1cr0cart";

//Set up sever on 8080 port (TCP Server)
WiFiServer server(8080);

//Set up clients
WiFiClient serverClients[MAX_SRV_CLIENTS];

//TCP and Serial buffer
int tcp_buf_size = 2048;
byte* tcp_buf = (byte*) malloc(tcp_buf_size);
int ser_buf_size = 1024;
byte* serial_buf = (byte*) malloc(ser_buf_size);

//Setup for AP
IPAddress ip(192, 168, 1, 1);
IPAddress gateway(198, 168, 1, 1);
IPAddress subnet(255, 255, 255, 0);

void setup() {
  //start serial
  Serial.begin(1000000);
  Serial.setRxBufferSize(1024);

#if DEBUG_PRINT == 1
  Serial.printf("Setting soft-AP configuration ... \r\n");
  Serial.printf("%s\r\n", WiFi.softAPConfig(ip, gateway, subnet) ? "Ready" : "Failed!");
#else
  WiFi.softAPConfig(ip, gateway, subnet);
#endif

  //Configure for access point.
#if DEBUG_PRINT == 1
  Serial.printf("Setting mode ... \r\n");
  Serial.printf("%s\r\n", WiFi.mode(WIFI_AP) ? "Ready" : "Failed!");
#else
  WiFi.mode(WIFI_AP);
#endif

#if DEBUG_PRINT == 1
  Serial.printf("Setting soft-AP ... \r\n");
#endif
  // Wait for Access point to be created
  while (!WiFi.softAP(ssid, password)) {
#if DEBUG_PRINT == 1
    Serial.printf("Failed!\r\n");
#endif
    delay(500); // Wait half a second before re-trying
  }
#if DEBUG_PRINT == 1
  Serial.printf("Ready\r\n");
#endif

#if DEBUG_PRINT == 1
  Serial.printf("Soft-AP IP address = ");
  Serial.print(WiFi.softAPIP());
  Serial.printf("\r\n");

  Serial.printf("MAC address = %s\r\n", WiFi.softAPmacAddress().c_str());
#endif

  //start server
  server.begin();
  server.setNoDelay(true);

}

void loop() {
  
  uint8_t i, j;
  //check if there are any new clients
  if (server.hasClient()){
    for(i = 0; i < MAX_SRV_CLIENTS; i++){
      //find free/disconnected spot
      if (!serverClients[i] || !serverClients[i].connected()){
        if(serverClients[i]) serverClients[i].stop();
        serverClients[i] = server.available();
        continue;
      }
    }
    serialFlush();
    //no free/disconnected spot so reject
    WiFiClient serverClient = server.available();
    serverClient.stop();
  }
  
  //check clients for data
  if (serverClients[0] && serverClients[0].connected()){
    //delay(1);
    while(serverClients[0].available()){
      int aval = std::min(serverClients[0].available(), tcp_buf_size);
      
      int retval = serverClients[0].read(tcp_buf, aval);
#if COMM_LOOP == 1     
      int written = serverClients[0].write((const uint8_t*)tcp_buf, aval);
#endif 
      Serial.write(tcp_buf, aval);
    }
  }
  
  if (Serial.available()) {
    size_t len = std::min(Serial.available(), ser_buf_size);
    Serial.readBytes(serial_buf, len);
    int written = serverClients[0].write((const uint8_t*)serial_buf, len);

#if SERIAL_LOOP == 1
    Serial.write(serial_buf, len);
#endif
  }
}

void serialFlush(){
  while(Serial.available() > 0) {
    char t = Serial.read();
  }
}
